@@ -23,7 +23,8 @@ pub struct View {
     pub sort_info: SortInfo,
     pub visible_pids: Vec<i32>,
     pub auxiliary_pids: Vec<i32>,
-    pub ppids: HashMap<i32, i32>,
+    pub parent_pids: HashMap<i32, i32>,
+    pub child_pids: HashMap<i32, Vec<i32>>,
 }
 
 impl View {
@@ -50,6 +51,7 @@ impl View {
                     align: ConfigColumnAlign::Left,
                     max_width: None,
                     min_width: None,
+                    visible: true,
                 });
             }
         }
@@ -57,28 +59,41 @@ impl View {
         let mut only_kind_found = false;
 
         for c in &config.columns {
-            let kind = match &c.kind {
+            let kinds = match &c.kind {
                 ConfigColumnKind::Slot => {
-                    let kind = if let Some(insert) = opt.insert.get(slot_idx) {
-                        find_column_kind(insert)
+                    let kinds = if let Some(insert) = opt.insert.get(slot_idx) {
+                        find_column_kind(insert).into_iter().collect()
                     } else {
-                        None
+                        vec![]
                     };
                     slot_idx += 1;
-                    kind
+                    kinds
                 }
-                x => Some(x.clone()),
+                ConfigColumnKind::MultiSlot => {
+                    let mut kinds = vec![];
+                    while let Some(insert) = opt.insert.get(slot_idx) {
+                        if let Some(kind) = find_column_kind(insert) {
+                            kinds.push(kind);
+                        }
+                        slot_idx += 1;
+                    }
+                    kinds
+                }
+                x => vec![x.clone()],
             };
 
-            if let Some(kind) = kind {
-                if let Some(ref only) = opt.only {
+            for kind in kinds {
+                let visible = if let Some(ref only) = opt.only {
                     let kind_name = KIND_LIST[&kind].0.to_lowercase();
                     if kind_name.find(&only.to_lowercase()).is_none() {
-                        continue;
+                        false
                     } else {
                         only_kind_found = true;
+                        true
                     }
-                }
+                } else {
+                    true
+                };
 
                 let column = gen_column(
                     &kind,
@@ -98,11 +113,16 @@ impl View {
                         align: c.align.clone(),
                         max_width: c.max_width,
                         min_width: c.min_width,
+                        visible,
                     });
                 }
             }
         }
 
+        if slot_idx < opt.insert.len() {
+            bail!("There is not enough slot for inserting columns {:?}.\nPlease add \"Slot\" or \"MultiSlot\" to your config.\nhttps://github.com/dalance/procs#insert-column", opt.insert);
+        }
+
         if let Some(ref only_kind) = opt.only {
             if !only_kind_found {
                 bail!("kind \"{}\" is not found in columns", only_kind);
@@ -124,9 +144,17 @@ impl View {
             }
         }
 
-        let mut ppids = HashMap::new();
-        for p in &proc {
-            ppids.insert(p.pid, p.ppid);
+        let mut parent_pids = HashMap::new();
+        let mut child_pids = HashMap::<i32, Vec<i32>>::new();
+        if opt.tree {
+            for p in &proc {
+                parent_pids.insert(p.pid, p.ppid);
+                if let Some(x) = child_pids.get_mut(&p.ppid) {
+                    x.push(p.pid);
+                } else {
+                    child_pids.insert(p.ppid, vec![p.pid]);
+                }
+            }
         }
 
         let term_info = TermInfo::new(clear_by_line);
@@ -142,7 +170,8 @@ impl View {
             sort_info,
             visible_pids: vec![],
             auxiliary_pids: vec![],
-            ppids,
+            parent_pids,
+            child_pids,
         })
     }
 
@@ -213,9 +242,14 @@ impl View {
         if opt.tree {
             let mut additional_pids = Vec::new();
             for pid in &candidate_pids {
-                let mut ppids = vec![];
-                self.get_ppids(*pid, &mut ppids);
-                additional_pids.append(&mut ppids);
+                let mut buf = vec![];
+                if config.display.show_parent_in_tree {
+                    self.get_parent_pids(*pid, &mut buf);
+                }
+                if config.display.show_children_in_tree {
+                    self.get_child_pids(*pid, &mut buf);
+                }
+                additional_pids.append(&mut buf);
             }
             let mut additional_pids: Vec<_> = additional_pids
                 .iter()
@@ -241,11 +275,22 @@ impl View {
         self.auxiliary_pids = auxiliary_pids;
     }
 
-    fn get_ppids(&self, pid: i32, ppids: &mut Vec<i32>) {
-        if let Some(x) = self.ppids.get(&pid) {
-            if !ppids.contains(x) {
-                ppids.push(*x);
-                self.get_ppids(*x, ppids);
+    fn get_parent_pids(&self, pid: i32, parent_pids: &mut Vec<i32>) {
+        if let Some(x) = self.parent_pids.get(&pid) {
+            if !parent_pids.contains(x) {
+                parent_pids.push(*x);
+                self.get_parent_pids(*x, parent_pids);
+            }
+        }
+    }
+
+    fn get_child_pids(&self, pid: i32, child_pids: &mut Vec<i32>) {
+        if let Some(pids) = self.child_pids.get(&pid) {
+            for x in pids {
+                if !child_pids.contains(x) {
+                    child_pids.push(*x);
+                    self.get_child_pids(*x, child_pids);
+                }
             }
         }
     }
@@ -358,21 +403,23 @@ impl View {
     fn display_header(&self, config: &Config, theme: &ConfigTheme) -> Result<(), Error> {
         let mut row = String::from("");
         for (i, c) in self.columns.iter().enumerate() {
-            let order = if i == self.sort_info.idx {
-                Some(self.sort_info.order.clone())
-            } else {
-                None
-            };
-            row = format!(
-                "{} {}",
-                row,
-                apply_color(
-                    c.column.display_header(&c.align, order, config),
-                    &config.style.header,
-                    theme,
-                    false
-                )
-            );
+            if c.visible {
+                let order = if i == self.sort_info.idx {
+                    Some(self.sort_info.order.clone())
+                } else {
+                    None
+                };
+                row = format!(
+                    "{} {}",
+                    row,
+                    apply_color(
+                        c.column.display_header(&c.align, order, config),
+                        &config.style.header,
+                        theme,
+                        false
+                    )
+                );
+            }
         }
         row = row.trim_end().to_string();
         row = truncate(&row, self.term_info.width).to_string();
@@ -383,16 +430,18 @@ impl View {
     fn display_unit(&self, config: &Config, theme: &ConfigTheme) -> Result<(), Error> {
         let mut row = String::from("");
         for c in &self.columns {
-            row = format!(
-                "{} {}",
-                row,
-                apply_color(
-                    c.column.display_unit(&c.align),
-                    &config.style.unit,
-                    theme,
-                    false
-                )
-            );
+            if c.visible {
+                row = format!(
+                    "{} {}",
+                    row,
+                    apply_color(
+                        c.column.display_unit(&c.align),
+                        &config.style.unit,
+                        theme,
+                        false
+                    )
+                );
+            }
         }
         row = row.trim_end().to_string();
         row = truncate(&row, self.term_info.width).to_string();
@@ -409,17 +458,19 @@ impl View {
     ) -> Result<(), Error> {
         let mut row = String::from("");
         for c in &self.columns {
-            row = format!(
-                "{} {}",
-                row,
-                apply_style(
-                    c.column.display_content(pid, &c.align).unwrap(),
-                    &c.style,
-                    &config.style,
-                    theme,
-                    auxiliary
-                )
-            );
+            if c.visible {
+                row = format!(
+                    "{} {}",
+                    row,
+                    apply_style(
+                        c.column.display_content(pid, &c.align).unwrap(),
+                        &c.style,
+                        &config.style,
+                        theme,
+                        auxiliary
+                    )
+                );
+            }
         }
         row = row.trim_end().to_string();
         row = truncate(&row, self.term_info.width).to_string();
